from __future__ import annotations
from typing import TYPE_CHECKING, List, Union
from pybi.utils.markdown2 import markdown


from pybi.core.components import ComponentTag
from .base import ReactiveComponent


if TYPE_CHECKING:
    from pybi.core.sql import SqlInfo


class Markdown(ReactiveComponent):
    def __init__(self, contexts: List[Union[str, SqlInfo]]) -> None:
        super().__init__(ComponentTag.Markdown)
        self.contexts = contexts

    def _to_json_dict(self):
        data = super()._to_json_dict()

        # data["html"] = markdown(
        #     self._md, extras=["fenced-code-blocks", "task_list", "code-color"]
        # )
        return data



from __future__ import annotations
from typing import TYPE_CHECKING, Dict, Tuple, List, Any, Union

from pybi.core.components.reactiveComponent.echarts import EChartUpdateInfo
from pybi.core.dataSource import DataSourceTable
import pybi.utils.echarts_opts_utils as echarts_opts_utils
import copy
from .utils import merge_user_options


if TYPE_CHECKING:
    pass


class BaseChart:
    def __init__(self) -> None:
        self.__base_opt = {
            "legend": [{}],
            "series": [],
            "title": [{}],
            "grid": [{"containLabel": True}],
            "tooltip": [{}],
        }
        self._merge_opt = {}
        self._updateInfos: List[EChartUpdateInfo] = []
        self._prop_by_paths: List[Tuple[str, Any]] = []

    def __add__(self, other: BaseChart):
        return ChartCollector().append(self).append(other)

    def merge(self, options: Dict):
        list_item_opts = {
            key: value if isinstance(value, List) or key == "series" else [value]
            for key, value in options.items()
        }

        self._merge_opt.update(list_item_opts)
        return self

    def get_options(self):
        return copy.deepcopy(self.__base_opt)

    def set_title(self, text: str):
        self.merge({"title": {"text": text}})

        return self

    def hover_filter(
        self, value_type: str, table: Union[str, DataSourceTable], field: str
    ):
        """
        value_type: Literal["x", "y", "value","color","name"]
        """
        if isinstance(table, DataSourceTable):
            table = table.source_name

        self._updateInfos.append(EChartUpdateInfo("hover", value_type, table, field))

        return self

    def click_filter(
        self, value_type: str, table: Union[str, DataSourceTable], field: str
    ):
        """
        value_type: Literal["x", "y", "value","color","name"]
        """
        if isinstance(table, DataSourceTable):
            table = table.source_name

        self._updateInfos.append(EChartUpdateInfo("click", value_type, table, field))

        return self

    def _post_options(self, opts: Dict):
        # opts = merge_user_options(opts, self._merge_opt)
        self._update_props_by_path(opts)
        return opts

    def _remove_filters(self, actionType: str):
        """
        actionType : 'click' | 'hover'
        """
        self._updateInfos = [
            info for info in self._updateInfos if info.actionType != actionType
        ]
        return self

    def remove_all_click_filter(self):
        return self._remove_filters("click")

    def _create_default_click_filter(self):
        pass

    def _create_options_ex(self):
        raise NotImplementedError

    def get_options_infos(
        self,
    ) -> Tuple[Dict, List[EChartUpdateInfo], List[str]]:
        opts, updateInfos, mapIds = self._create_options_ex()
        opts = self._post_options(opts)
        return opts, updateInfos, mapIds

    def set_prop_by_path(self, path: str, value):
        """
        >>> .set_prop_by_path('xAxis[0].axisLabel.interval',1)
        """
        self._prop_by_paths.append((path, value))
        return self

    def _update_props_by_path(self, opts: Dict):
        for path, value in self._prop_by_paths:
            echarts_opts_utils.set_prop_by_path(opts, path, value)


class ChartCollector:
    def __init__(self) -> None:
        self._collector: List[BaseChart] = []

    def append(self, other: BaseChart):
        self._collector.append(other)
        return self

    def __add__(self, other: Union[ChartCollector, BaseChart]):
        if isinstance(other, BaseChart):
            other = ChartCollector().append(other)

        self._collector.extend(other._collector)
        return self



from pybi.core.components.component import Component
from .componentTag import ComponentTag
import re
import pybi as pbi

from typing import Optional, Union


class TextComponent(Component):
    def __init__(self, content: str) -> None:
        super().__init__(ComponentTag.Text)
        self.content = content


class UploadComponent(Component):
    def __init__(self) -> None:
        super().__init__(ComponentTag.Upload)


class SvgIconComponent(Component):
    replace_svg_size_pat = re.compile(r"(width|height)=.+?\s", re.I | re.DOTALL)

    def __init__(self, svg: str, size: str, color: str) -> None:
        super().__init__(ComponentTag.SvgIcon)

        svg = SvgIconComponent.replace_svg_size_pat.sub("", svg)
        self.svg = svg
        self.size = size
        self.color = color


class IconComponent(Component):
    def __init__(self, icon_id: str, size: str, color: str) -> None:
        super().__init__(ComponentTag.Icon)

        self.iconID = icon_id
        self.size = size
        self.color = color


class SpaceComponent(Component):
    def __init__(self, space: Optional[str] = None, auto_fill_row=False) -> None:
        super().__init__(ComponentTag.Space)
        self._space = space
        self._auto_fill_row = auto_fill_row

    # def __mul__(self, num: int) -> None:
    #     assert isinstance(num, int), "other must be int"

    #     if num < 1:
    #         raise ValueError("num must be greater than 0")

    #     if num == 1:
    #         return

    #     for _ in range(num):
    #         pbi.space(self.space)

    def _to_json_dict(self):
        data = super()._to_json_dict()
        if not self._space is None:
            data["space"] = self._space

        if self._auto_fill_row:
            data["autoFillRow"] = self._auto_fill_row

        return data



from pybi.core.styles.styles import StyleBuilder
class TextSize:
	@property
	def text_xs(self):
		return StyleBuilder({"font-size": " 0.75rem", "line-height": " 1rem"})
	@property
	def text_sm(self):
		return StyleBuilder({"font-size": " 0.875rem", "line-height": " 1.25rem"})
	@property
	def text_base(self):
		return StyleBuilder({"font-size": " 1rem", "line-height": " 1.5rem"})
	@property
	def text_lg(self):
		return StyleBuilder({"font-size": " 1.125rem", "line-height": " 1.75rem"})
	@property
	def text_xl(self):
		return StyleBuilder({"font-size": " 1.25rem", "line-height": " 1.75rem"})
	@property
	def text_2xl(self):
		return StyleBuilder({"font-size": " 1.5rem", "line-height": " 2rem"})
	@property
	def text_3xl(self):
		return StyleBuilder({"font-size": " 1.875rem", "line-height": " 2.25rem"})
	@property
	def text_4xl(self):
		return StyleBuilder({"font-size": " 2.25rem", "line-height": " 2.5rem"})
	@property
	def text_5xl(self):
		return StyleBuilder({"font-size": " 3rem", "line-height": " 1"})
	@property
	def text_6xl(self):
		return StyleBuilder({"font-size": " 3.75rem", "line-height": " 1"})
	@property
	def text_7xl(self):
		return StyleBuilder({"font-size": " 4.5rem", "line-height": " 1"})
	@property
	def text_8xl(self):
		return StyleBuilder({"font-size": " 6rem", "line-height": " 1"})
	@property
	def text_9xl(self):
		return StyleBuilder({"font-size": " 8rem", "line-height": " 1"})



from __future__ import annotations
from typing import TYPE_CHECKING, List, Optional, Dict, Set, Any, Union

from pybi.utils.data_gen import Jsonable

from pybi.core.components import ComponentTag
from pybi.core.components.component import Component
from pybi.core.sql import SqlInfo

from pybi.core.dataSource import DataSourceView, DataSourceTable
import re


if TYPE_CHECKING:
    from pybi.core.dataSource import DataSourceTable


class UpdateInfo(Jsonable):
    def __init__(self, table: str, field: str) -> None:
        super().__init__()
        self.table = table
        self.field = field


class ReactiveComponent(Component):
    def __init__(self, tag: ComponentTag) -> None:
        super().__init__(tag)
        self._updateInfos: List[UpdateInfo] = []

    def add_updateInfo(self, table: str, field: str):
        self._updateInfos.append(UpdateInfo(table, field))
        return self

    def _to_json_dict(self):
        data = super()._to_json_dict()
        data["updateInfos"] = self._updateInfos

        return data


class SingleReactiveComponent(ReactiveComponent):
    def __init__(self, tag: ComponentTag, sql: SqlInfo) -> None:
        super().__init__(tag)
        self.sqlInfo = sql

    # def sql(self, sql: str):
    #     self._sql = sql
    #     return self



from __future__ import annotations


from .base import BaseChart
import pybi as pbi
import pybi.utils.sql as sqlUtils


from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from pybi.core.dataSource import DataSourceTable


class RadarChart(BaseChart):
    def __init__(
        self,
        data: DataSourceTable,
        indicator: str,
        name: str,
        value: str,
        agg="round(avg(${}),2)",
    ):
        super().__init__()
        self.data = data
        self.indicator = indicator
        self.value = value
        self.name = name
        self.agg = agg
        self._series_configs = {}

    def _create_default_click_filter(self):
        pass

    def _create_options_ex(self):
        opts = super().get_options()

        agg_field = f"{sqlUtils.apply_agg(self.agg, self.value)} as value"

        dv_summary = pbi.set_dataView(
            f"select {self.name} as name,{self.indicator} as indicator,{agg_field} from {self.data} group by {self.name},{self.indicator}"
        )

        dv_indicator = pbi.set_dataView(
            f"""select indicator,max(value) as max,
                    row_number() OVER (ORDER BY indicator) as row_num
            from {dv_summary} 
            group by indicator"""
        )

        dv_idc_name = pbi.set_dataView(
            f"""
        select t1.indicator,t1.row_num ,t2.name
        from {dv_indicator} as t1
        cross join (select distinct name from {dv_summary}) as t2
        """
        )

        dv_data = pbi.set_dataView(
            f"""select t1.indicator, t1.name,t2.value
            from {dv_idc_name} as t1 
            left join {dv_summary} as t2 
            on t1.indicator=t2.indicator and t1.name=t2.name 
            order by t1.name,t1.row_num
            """
        )

        base_opt = {
            "tooltip": {},
            "series": [],
            "radar": {
                "indicator": pbi.sql(
                    f"select indicator as name,max from {dv_indicator}"
                ).js_map("return rows")
            },
        }

        base_opt.update(opts)

        data_query = pbi.sql(f"select * from {dv_data}").split_group(
            "name",
            """
const value = rows.map(r=> r.value)
return {name:key,value}      
                """,
        )

        base_opt["series"].append(
            {
                "type": "radar",
                "data": data_query,
                # "universalTransition": {"enabled": True, "divideShape": "clone"},
                "areaStyle": {"opacity": 0.1},
            }
        )

        return base_opt, self._updateInfos, []



# from .styles import *
from .tailwindStyles.textColor import TextColor
from .tailwindStyles.boxShadow import BoxShadow
from .tailwindStyles.textSize import TextSize
from .tailwindStyles.textAlign import TextAlign

from functools import lru_cache

__all__ = ["styles"]


class StylesMeta:
    @property
    @lru_cache(1)
    def textColor(self):
        return TextColor()

    @property
    @lru_cache(1)
    def boxShadow(self):
        return BoxShadow()

    @property
    @lru_cache(1)
    def textSize(self):
        return TextSize()

    @property
    @lru_cache(1)
    def textAlign(self):
        return TextAlign()


styles = StylesMeta()


# text_color = textColor



from __future__ import annotations
from typing import List, Union, TYPE_CHECKING


from pybi.core.components import ComponentTag
from .base import ReactiveComponent


if TYPE_CHECKING:
    from pybi.core.sql import SqlInfo, ForeachRowInfo


class TextValue(ReactiveComponent):
    def __init__(self, contexts: List[Union[str, SqlInfo, ForeachRowInfo]]) -> None:
        super().__init__(ComponentTag.TextValue)
        self.contexts = contexts



from pybi.core.styles.styles import StyleBuilder
class BoxShadow:
	@property
	def shadow_sm(self):
		return StyleBuilder({"box-shadow": " 0 1px 2px 0 rgb(0 0 0 / 0.05)"})
	@property
	def shadow(self):
		return StyleBuilder({"box-shadow": " 0 1px 3px 0 rgb(0 0 0 / 0.1), 0 1px 2px -1px rgb(0 0 0 / 0.1)"})
	@property
	def shadow_md(self):
		return StyleBuilder({"box-shadow": " 0 4px 6px -1px rgb(0 0 0 / 0.1), 0 2px 4px -2px rgb(0 0 0 / 0.1)"})
	@property
	def shadow_lg(self):
		return StyleBuilder({"box-shadow": " 0 10px 15px -3px rgb(0 0 0 / 0.1), 0 4px 6px -4px rgb(0 0 0 / 0.1)"})
	@property
	def shadow_xl(self):
		return StyleBuilder({"box-shadow": " 0 20px 25px -5px rgb(0 0 0 / 0.1), 0 8px 10px -6px rgb(0 0 0 / 0.1)"})
	@property
	def shadow_2xl(self):
		return StyleBuilder({"box-shadow": " 0 25px 50px -12px rgb(0 0 0 / 0.25)"})
	@property
	def shadow_inner(self):
		return StyleBuilder({"box-shadow": " inset 0 2px 4px 0 rgb(0 0 0 / 0.05)"})
	@property
	def shadow_none(self):
		return StyleBuilder({"box-shadow": " 0 0 #0000"})



from __future__ import annotations
from typing import TYPE_CHECKING, Dict, List, Optional, Callable
import pybi.utils.data_gen as data_gen
import jinja2
from pathlib import Path

# TODO:
_pj_root = data_gen.get_project_root()
_static_path = _pj_root / "static"


def _get_file_path_from_static(file_name: str):
    return str(_static_path / file_name)


def js_online_resource(src: str):
    def fn():
        return f'<script src="{src}"></script>'

    return fn


def js_offline_resource(path: str, id=""):
    def fn():
        return f'<script id="{id}">{Path(path).read_text("utf8")}</script>'

    return fn


def css_online_resource(href: str):
    def fn():
        return f'<link rel="stylesheet" type="text/css" href="{href}">'

    return fn


def css_offline_resource(path: str, id=""):
    def fn():
        return f'<style id="{id}">{Path(path).read_text("utf8")}</style>'

    return fn


def empty_resource():
    def fn():
        return ""

    return fn


TResource = Callable


_system_resources: Dict[str, TResource] = {
    "sysApp": js_offline_resource(
        _get_file_path_from_static("sysApp.iife.js"), id="sysApp"
    ),
    "sysApp-css": css_offline_resource(
        _get_file_path_from_static("sysApp-style.css"), id="sysApp-css"
    ),
    "vue-js": js_offline_resource(
        _get_file_path_from_static("vue.global.prod.min.js"), id="vue-js"
    ),
    "echarts-js": js_offline_resource(
        _get_file_path_from_static("echarts.min.js"), id="echarts-js"
    ),
}


_env = jinja2.Environment(loader=jinja2.PackageLoader("pybi", "template"))
_html_template = _env.get_template("index.html")


class ResourceManager:
    def __init__(self):
        pass
        # self.system_resources =
        self.sysApp_css = _system_resources["sysApp-css"]
        self.sysApp_js = _system_resources["sysApp"]
        self.vue_js = _system_resources["vue-js"]

        self.echarts_core_js = empty_resource()
        self.echarts_cps_js = empty_resource()
        self.echarts_cps_css = empty_resource()

        self.element_cps_js = empty_resource()
        self.element_cps_css = empty_resource()

        self.mermaid_cps_js = empty_resource()
        self.mermaid_cps_css = empty_resource()

        self.experimental_cps_js = empty_resource()

        self.zip_js = None

    def register_echarts(self):
        self.echarts_cps_js = js_offline_resource(
            _get_file_path_from_static("echartsCps.iife.js"), id="echartsCps-js"
        )
        self.echarts_cps_css = css_offline_resource(
            _get_file_path_from_static("echartsCps-style.css"), id="echartsCps-style"
        )
        self.echarts_core_js = js_offline_resource(
            _get_file_path_from_static("echarts.min.js"), id="echarts-js"
        )
        return self

    def register_element_cps(self):
        self.element_cps_js = js_offline_resource(
            _get_file_path_from_static("elementCps.iife.js"), id="elementCps-js"
        )
        self.element_cps_css = css_offline_resource(
            _get_file_path_from_static("elementCps-style.css"), id="elementCps-style"
        )
        return self

    def register_experimental_cps(self):
        self.experimental_cps_js = js_offline_resource(
            _get_file_path_from_static("experimentalCps.iife.js"),
            id="experimentalCps-js",
        )
        # self.experimental_cps_css = css_offline_resource(
        #     _get_file_path_from_static("experimentalCps-style.css"),
        #     id="experimentalCps-style",
        # )
        return self

    def register_mermaid_cps(self):
        self.mermaid_cps_js = js_offline_resource(
            _get_file_path_from_static("mermaidCps.iife.js"), id="mermaidCps-js"
        )

        self.mermaid_cps_css = css_offline_resource(
            _get_file_path_from_static("mermaidCps-style.css"), id="mermaidCps-style"
        )

        return self

    def build_html(self, appConfig: str, svg_infos: List[Dict]) -> str:
        res = _html_template.render(
            app_config=appConfig, resources=self, svg_infos=svg_infos
        )
        assert isinstance(res, str)
        return res



from __future__ import annotations
from typing import TYPE_CHECKING, List, Dict


from pybi.core.components import ComponentTag
from .base import SingleReactiveComponent


if TYPE_CHECKING:
    from pybi.core.sql import SqlInfo


class Checkbox(SingleReactiveComponent):
    def __init__(self, sql: SqlInfo) -> None:
        super().__init__(ComponentTag.Checkbox, sql)
        self.title = ""
        self.mode = "tile"
        self.multiple = True
        self.itemDirection = "row"
        self.__hidden_title = None

    def set_itemDirection(self, value="row"):
        """选项排列方向

        Args:
            value (str, optional): 'row' | 'column' . Defaults to "row".
        """
        self.itemDirection = value
        return self

    def set_width(self, value: str):
        """设置宽度

        Args:
            value (str): css width的可能值.[参考文档](https://developer.mozilla.org/zh-CN/docs/Web/CSS/width)

        ## 示例
        ```python
        # 200px , 20em, 20vw
        pbi.add_checkbox(data["name"]).set_width('200px')
        ```
        """
        self.set_style(f"width:{value}")
        return self

    def set_height(self, value: str):
        """设置高度

        Args:
            value (str): css width的可能值.[参考文档](https://developer.mozilla.org/zh-CN/docs/Web/CSS/height)

        ## 示例
        ```python
        # 200px , 20em, 20vh
        pbi.add_checkbox(data["name"]).set_height('200px')
        ```

        """
        self.set_style(f"height:{value}")
        return self

    def set_mode(self, mode="tile"):
        """选项显示模式

        Args:
            mode (str, optional): 'tile' | 'list'. Defaults to 'tile'.
            'tile':磁贴模式,选项以磁贴方式显示
            'list':普通checkbox方式

        """
        self.mode = mode
        return self

    def set_hiddenTitle(self, value=True):
        """
        不显示标题
        """
        self.__hidden_title = value
        return self

    def set_title(self, title: str):
        self.title = title
        return self

    def set_multiple(self, multiple: bool):
        self.multiple = multiple
        return self

    def set_props(self, props: Dict):
        """
        [checkbox props](https://element-plus.org/zh-CN/component/checkbox.html#checkboxgroup-attributes)
        e.g
        >>> .add_checkbox(...).set_props({'min':2})
        """
        return super().set_props(props)

    def _to_json_dict(self):
        data = super()._to_json_dict()
        if self.__hidden_title:
            data["hiddenTitle"] = self.__hidden_title

        return data



from pybi.core.styles.styles import StyleBuilder
class TextAlign:
	@property
	def text_left(self):
		return StyleBuilder({"text-align": " left"})
	@property
	def text_center(self):
		return StyleBuilder({"text-align": " center"})
	@property
	def text_right(self):
		return StyleBuilder({"text-align": " right"})
	@property
	def text_justify(self):
		return StyleBuilder({"text-align": " justify"})
	@property
	def text_start(self):
		return StyleBuilder({"text-align": " start"})
	@property
	def text_end(self):
		return StyleBuilder({"text-align": " end"})



from __future__ import annotations
from typing import TYPE_CHECKING, Dict, Any, Optional, Union


from pybi.core.components import ComponentTag
from .base import SingleReactiveComponent


if TYPE_CHECKING:
    from pybi.core.sql import SqlInfo


_TColumnprops = Dict[str, Dict[str, Any]]


class Table(SingleReactiveComponent):
    def __init__(
        self,
        sql: SqlInfo,
    ) -> None:
        super().__init__(ComponentTag.Table, sql)
        self.pageSize = 10
        self.tableHeight = "initial"
        self.tableWidth = "initial"
        self.column_props: _TColumnprops = {}
        self._showCopyButton: Optional[bool] = None

    def show_copy_button(self):
        self._showCopyButton = True
        return self

    def set_page_size(self, size: int):
        """
        设置表格每页行数
        size: >=5 ,默认10
        """
        self.pageSize = max(size, 5)
        return self

    def set_table_height(self, height="initial"):
        """
        表格高度
        height: 'initial'(默认值),'30em','30%','30vh'
        如果设置为initial,则表格会以展示一页所有数据的高度作为固定高度
        """
        self.tableHeight = height
        return self

    def set_table_width(self, width="initial"):
        """
        表格高度
        width: 'initial'(默认值),'30em','30%','30vh'
        """
        self.tableWidth = width
        return self

    def set_props(self, props: Dict):
        """设置表格属性。可配置的属性参考[Table 属性](https://element-plus.org/zh-CN/component/table.html#table-%E5%B1%9E%E6%80%A7)

        Args:
            props (Dict): 属性键值对

        ### 使用
        >>>
        ```python
        pbi.add_table(dv1).set_props({"show-summary": True})
        ```
        """
        return super().set_props(props)

    def set_column_props(self, props: _TColumnprops):
        """配置每列的属性。可配置的属性参考[table columns 文档](https://element-plus.org/zh-CN/component/table.html#table-column-%E5%B1%9E%E6%80%A7)

        Args:
            props (_TColumnprops): 每列的配置项。格式:`{列名:配置项字典}`


        ### 使用
        >>>
        ```python
        col_props = {
            "日期": {"width": "500px", "sortable": True},
            "计数单位": {"width": "200px", "sortable": True},
        }
        pbi.add_table(dv1).set_column_props(col_props)
        ```
        """
        self.column_props.update(props)
        return self

    def _to_json_dict(self):
        data = super()._to_json_dict()

        if len(self.column_props):
            data["columnProps"] = self.column_props

        if self._showCopyButton:
            data["showCopyButton"] = self._showCopyButton

        return data



from typing import TYPE_CHECKING, Dict, Sequence
import pybi.utils.echarts_opts_utils as echarts_opts_utils


def pyecharts_json_default(o, utils, basicOptsType):

    if isinstance(o, utils.JsCode):
        return (
            o.replace("\\n|\\t", "").replace(r"\\n", "\n").replace(r"\\t", "\t").js_code
        )
    if isinstance(o, basicOptsType):
        if isinstance(o.opts, Sequence):
            return [utils.remove_key_with_none_value(item) for item in o.opts]
        else:
            return utils.remove_key_with_none_value(o.opts)


def replace_jscode(options: Dict):
    from pyecharts.commons import utils as pyecharts_utils
    from pyecharts.options.series_options import BasicOpts

    iteror = echarts_opts_utils.iter_each_items(options)

    new_obj = None

    try:
        while 1:
            target, path = iteror.send(new_obj)
            new_obj = None
            if isinstance(target, pyecharts_utils.JsCode):
                code_str = (
                    target.replace("\\n|\\t", "")
                    .replace(r"\\n", "\n")
                    .replace(r"\\t", "\t")
                    .js_code
                )

                echarts_opts_utils.set_prop_by_path(options, path, code_str)

            if isinstance(target, BasicOpts):
                if isinstance(target.opts, Sequence):
                    new_obj = [
                        pyecharts_utils.remove_key_with_none_value(item)
                        for item in target.opts
                    ]
                    echarts_opts_utils.set_prop_by_path(options, path, new_obj)

                else:
                    new_obj = pyecharts_utils.remove_key_with_none_value(target.opts)
                    echarts_opts_utils.set_prop_by_path(options, path, new_obj)
    except StopIteration:
        pass

    # for target, path in iteror:
    #     if isinstance(target, pyecharts_utils.JsCode):
    #         code_str = (
    #             target.replace("\\n|\\t", "")
    #             .replace(r"\\n", "\n")
    #             .replace(r"\\t", "\t")
    #             .js_code
    #         )

    #         echarts_opts_utils.set_prop_by_path(options, path, code_str)

    #     if isinstance(target, BasicOpts):
    #         if isinstance(target.opts, Sequence):
    #             new_obj = [
    #                 pyecharts_utils.remove_key_with_none_value(item)
    #                 for item in target.opts
    #             ]
    #             echarts_opts_utils.set_prop_by_path(options, path, new_obj)
    #             iteror.send(new_obj)
    #         else:
    #             new_obj = pyecharts_utils.remove_key_with_none_value(target.opts)
    #             echarts_opts_utils.set_prop_by_path(options, path, new_obj)
    #             iteror.send(new_obj)

    return options



import pybi.utils.data_gen as data_gen
from pathlib import Path
import simplejson as json

# TODO:
_pj_root = data_gen.get_project_root()
_static_path = _pj_root / "static"


def load_province_data():
    return json.loads(Path(_static_path / "province_map_full.json").read_bytes())


class WebResourceManager:
    def __init__(self) -> None:
        self.echarts_map_names = set()
        self.wrs = []

    def mark_echarts_map(self, map_name: str, online=False):
        if map_name in self.echarts_map_names:
            return

        base_wr = {
            "id": map_name,
            "type": "echarts-map",
        }

        if online:
            base_wr.update(
                **{
                    "input": None,
                    "actionPipe": [
                        {
                            "name": "fetch",
                            "args": {
                                "url": "https://geo.datav.aliyun.com/areas_v3/bound/100000_full.json",
                                "options": {},
                            },
                        }
                    ],
                }
            )
            return

        base_wr.update(
            **{
                "input": load_province_data(),
            }
        )

        self.wrs.append(base_wr)
        self.echarts_map_names.add(map_name)

    def create_webResources(self):
        return self.wrs



from pybi.core.styles.styles import StyleBuilder
class TextColor:
	@property
	def text_inherit(self):
		return StyleBuilder({"color": " inherit"})
	@property
	def text_current(self):
		return StyleBuilder({"color": " currentColor"})
	@property
	def text_transparent(self):
		return StyleBuilder({"color": " transparent"})
	@property
	def text_black(self):
		return StyleBuilder({"color": " rgb(0 0 0)"})
	@property
	def text_white(self):
		return StyleBuilder({"color": " rgb(255 255 255)"})
	@property
	def text_slate_50(self):
		return StyleBuilder({"color": " rgb(248 250 252)"})
	@property
	def text_slate_100(self):
		return StyleBuilder({"color": " rgb(241 245 249)"})
	@property
	def text_slate_200(self):
		return StyleBuilder({"color": " rgb(226 232 240)"})
	@property
	def text_slate_300(self):
		return StyleBuilder({"color": " rgb(203 213 225)"})
	@property
	def text_slate_400(self):
		return StyleBuilder({"color": " rgb(148 163 184)"})
	@property
	def text_slate_500(self):
		return StyleBuilder({"color": " rgb(100 116 139)"})
	@property
	def text_slate_600(self):
		return StyleBuilder({"color": " rgb(71 85 105)"})
	@property
	def text_slate_700(self):
		return StyleBuilder({"color": " rgb(51 65 85)"})
	@property
	def text_slate_800(self):
		return StyleBuilder({"color": " rgb(30 41 59)"})
	@property
	def text_slate_900(self):
		return StyleBuilder({"color": " rgb(15 23 42)"})
	@property
	def text_gray_50(self):
		return StyleBuilder({"color": " rgb(249 250 251)"})
	@property
	def text_gray_100(self):
		return StyleBuilder({"color": " rgb(243 244 246)"})
	@property
	def text_gray_200(self):
		return StyleBuilder({"color": " rgb(229 231 235)"})
	@property
	def text_gray_300(self):
		return StyleBuilder({"color": " rgb(209 213 219)"})
	@property
	def text_gray_400(self):
		return StyleBuilder({"color": " rgb(156 163 175)"})
	@property
	def text_gray_500(self):
		return StyleBuilder({"color": " rgb(107 114 128)"})
	@property
	def text_gray_600(self):
		return StyleBuilder({"color": " rgb(75 85 99)"})
	@property
	def text_gray_700(self):
		return StyleBuilder({"color": " rgb(55 65 81)"})
	@property
	def text_gray_800(self):
		return StyleBuilder({"color": " rgb(31 41 55)"})
	@property
	def text_gray_900(self):
		return StyleBuilder({"color": " rgb(17 24 39)"})
	@property
	def text_zinc_50(self):
		return StyleBuilder({"color": " rgb(250 250 250)"})
	@property
	def text_zinc_100(self):
		return StyleBuilder({"color": " rgb(244 244 245)"})
	@property
	def text_zinc_200(self):
		return StyleBuilder({"color": " rgb(228 228 231)"})
	@property
	def text_zinc_300(self):
		return StyleBuilder({"color": " rgb(212 212 216)"})
	@property
	def text_zinc_400(self):
		return StyleBuilder({"color": " rgb(161 161 170)"})
	@property
	def text_zinc_500(self):
		return StyleBuilder({"color": " rgb(113 113 122)"})
	@property
	def text_zinc_600(self):
		return StyleBuilder({"color": " rgb(82 82 91)"})
	@property
	def text_zinc_700(self):
		return StyleBuilder({"color": " rgb(63 63 70)"})
	@property
	def text_zinc_800(self):
		return StyleBuilder({"color": " rgb(39 39 42)"})
	@property
	def text_zinc_900(self):
		return StyleBuilder({"color": " rgb(24 24 27)"})
	@property
	def text_neutral_50(self):
		return StyleBuilder({"color": " rgb(250 250 250)"})
	@property
	def text_neutral_100(self):
		return StyleBuilder({"color": " rgb(245 245 245)"})
	@property
	def text_neutral_200(self):
		return StyleBuilder({"color": " rgb(229 229 229)"})
	@property
	def text_neutral_300(self):
		return StyleBuilder({"color": " rgb(212 212 212)"})
	@property
	def text_neutral_400(self):
		return StyleBuilder({"color": " rgb(163 163 163)"})
	@property
	def text_neutral_500(self):
		return StyleBuilder({"color": " rgb(115 115 115)"})
	@property
	def text_neutral_600(self):
		return StyleBuilder({"color": " rgb(82 82 82)"})
	@property
	def text_neutral_700(self):
		return StyleBuilder({"color": " rgb(64 64 64)"})
	@property
	def text_neutral_800(self):
		return StyleBuilder({"color": " rgb(38 38 38)"})
	@property
	def text_neutral_900(self):
		return StyleBuilder({"color": " rgb(23 23 23)"})
	@property
	def text_stone_50(self):
		return StyleBuilder({"color": " rgb(250 250 249)"})
	@property
	def text_stone_100(self):
		return StyleBuilder({"color": " rgb(245 245 244)"})
	@property
	def text_stone_200(self):
		return StyleBuilder({"color": " rgb(231 229 228)"})
	@property
	def text_stone_300(self):
		return StyleBuilder({"color": " rgb(214 211 209)"})
	@property
	def text_stone_400(self):
		return StyleBuilder({"color": " rgb(168 162 158)"})
	@property
	def text_stone_500(self):
		return StyleBuilder({"color": " rgb(120 113 108)"})
	@property
	def text_stone_600(self):
		return StyleBuilder({"color": " rgb(87 83 78)"})
	@property
	def text_stone_700(self):
		return StyleBuilder({"color": " rgb(68 64 60)"})
	@property
	def text_stone_800(self):
		return StyleBuilder({"color": " rgb(41 37 36)"})
	@property
	def text_stone_900(self):
		return StyleBuilder({"color": " rgb(28 25 23)"})
	@property
	def text_red_50(self):
		return StyleBuilder({"color": " rgb(254 242 242)"})
	@property
	def text_red_100(self):
		return StyleBuilder({"color": " rgb(254 226 226)"})
	@property
	def text_red_200(self):
		return StyleBuilder({"color": " rgb(254 202 202)"})
	@property
	def text_red_300(self):
		return StyleBuilder({"color": " rgb(252 165 165)"})
	@property
	def text_red_400(self):
		return StyleBuilder({"color": " rgb(248 113 113)"})
	@property
	def text_red_500(self):
		return StyleBuilder({"color": " rgb(239 68 68)"})
	@property
	def text_red_600(self):
		return StyleBuilder({"color": " rgb(220 38 38)"})
	@property
	def text_red_700(self):
		return StyleBuilder({"color": " rgb(185 28 28)"})
	@property
	def text_red_800(self):
		return StyleBuilder({"color": " rgb(153 27 27)"})
	@property
	def text_red_900(self):
		return StyleBuilder({"color": " rgb(127 29 29)"})
	@property
	def text_orange_50(self):
		return StyleBuilder({"color": " rgb(255 247 237)"})
	@property
	def text_orange_100(self):
		return StyleBuilder({"color": " rgb(255 237 213)"})
	@property
	def text_orange_200(self):
		return StyleBuilder({"color": " rgb(254 215 170)"})
	@property
	def text_orange_300(self):
		return StyleBuilder({"color": " rgb(253 186 116)"})
	@property
	def text_orange_400(self):
		return StyleBuilder({"color": " rgb(251 146 60)"})
	@property
	def text_orange_500(self):
		return StyleBuilder({"color": " rgb(249 115 22)"})
	@property
	def text_orange_600(self):
		return StyleBuilder({"color": " rgb(234 88 12)"})
	@property
	def text_orange_700(self):
		return StyleBuilder({"color": " rgb(194 65 12)"})
	@property
	def text_orange_800(self):
		return StyleBuilder({"color": " rgb(154 52 18)"})
	@property
	def text_orange_900(self):
		return StyleBuilder({"color": " rgb(124 45 18)"})
	@property
	def text_amber_50(self):
		return StyleBuilder({"color": " rgb(255 251 235)"})
	@property
	def text_amber_100(self):
		return StyleBuilder({"color": " rgb(254 243 199)"})
	@property
	def text_amber_200(self):
		return StyleBuilder({"color": " rgb(253 230 138)"})
	@property
	def text_amber_300(self):
		return StyleBuilder({"color": " rgb(252 211 77)"})
	@property
	def text_amber_400(self):
		return StyleBuilder({"color": " rgb(251 191 36)"})
	@property
	def text_amber_500(self):
		return StyleBuilder({"color": " rgb(245 158 11)"})
	@property
	def text_amber_600(self):
		return StyleBuilder({"color": " rgb(217 119 6)"})
	@property
	def text_amber_700(self):
		return StyleBuilder({"color": " rgb(180 83 9)"})
	@property
	def text_amber_800(self):
		return StyleBuilder({"color": " rgb(146 64 14)"})
	@property
	def text_amber_900(self):
		return StyleBuilder({"color": " rgb(120 53 15)"})
	@property
	def text_yellow_50(self):
		return StyleBuilder({"color": " rgb(254 252 232)"})
	@property
	def text_yellow_100(self):
		return StyleBuilder({"color": " rgb(254 249 195)"})
	@property
	def text_yellow_200(self):
		return StyleBuilder({"color": " rgb(254 240 138)"})
	@property
	def text_yellow_300(self):
		return StyleBuilder({"color": " rgb(253 224 71)"})
	@property
	def text_yellow_400(self):
		return StyleBuilder({"color": " rgb(250 204 21)"})
	@property
	def text_yellow_500(self):
		return StyleBuilder({"color": " rgb(234 179 8)"})
	@property
	def text_yellow_600(self):
		return StyleBuilder({"color": " rgb(202 138 4)"})
	@property
	def text_yellow_700(self):
		return StyleBuilder({"color": " rgb(161 98 7)"})
	@property
	def text_yellow_800(self):
		return StyleBuilder({"color": " rgb(133 77 14)"})
	@property
	def text_yellow_900(self):
		return StyleBuilder({"color": " rgb(113 63 18)"})
	@property
	def text_lime_50(self):
		return StyleBuilder({"color": " rgb(247 254 231)"})
	@property
	def text_lime_100(self):
		return StyleBuilder({"color": " rgb(236 252 203)"})
	@property
	def text_lime_200(self):
		return StyleBuilder({"color": " rgb(217 249 157)"})
	@property
	def text_lime_300(self):
		return StyleBuilder({"color": " rgb(190 242 100)"})
	@property
	def text_lime_400(self):
		return StyleBuilder({"color": " rgb(163 230 53)"})
	@property
	def text_lime_500(self):
		return StyleBuilder({"color": " rgb(132 204 22)"})
	@property
	def text_lime_600(self):
		return StyleBuilder({"color": " rgb(101 163 13)"})
	@property
	def text_lime_700(self):
		return StyleBuilder({"color": " rgb(77 124 15)"})
	@property
	def text_lime_800(self):
		return StyleBuilder({"color": " rgb(63 98 18)"})
	@property
	def text_lime_900(self):
		return StyleBuilder({"color": " rgb(54 83 20)"})
	@property
	def text_green_50(self):
		return StyleBuilder({"color": " rgb(240 253 244)"})
	@property
	def text_green_100(self):
		return StyleBuilder({"color": " rgb(220 252 231)"})
	@property
	def text_green_200(self):
		return StyleBuilder({"color": " rgb(187 247 208)"})
	@property
	def text_green_300(self):
		return StyleBuilder({"color": " rgb(134 239 172)"})
	@property
	def text_green_400(self):
		return StyleBuilder({"color": " rgb(74 222 128)"})
	@property
	def text_green_500(self):
		return StyleBuilder({"color": " rgb(34 197 94)"})
	@property
	def text_green_600(self):
		return StyleBuilder({"color": " rgb(22 163 74)"})
	@property
	def text_green_700(self):
		return StyleBuilder({"color": " rgb(21 128 61)"})
	@property
	def text_green_800(self):
		return StyleBuilder({"color": " rgb(22 101 52)"})
	@property
	def text_green_900(self):
		return StyleBuilder({"color": " rgb(20 83 45)"})
	@property
	def text_emerald_50(self):
		return StyleBuilder({"color": " rgb(236 253 245)"})
	@property
	def text_emerald_100(self):
		return StyleBuilder({"color": " rgb(209 250 229)"})
	@property
	def text_emerald_200(self):
		return StyleBuilder({"color": " rgb(167 243 208)"})
	@property
	def text_emerald_300(self):
		return StyleBuilder({"color": " rgb(110 231 183)"})
	@property
	def text_emerald_400(self):
		return StyleBuilder({"color": " rgb(52 211 153)"})
	@property
	def text_emerald_500(self):
		return StyleBuilder({"color": " rgb(16 185 129)"})
	@property
	def text_emerald_600(self):
		return StyleBuilder({"color": " rgb(5 150 105)"})
	@property
	def text_emerald_700(self):
		return StyleBuilder({"color": " rgb(4 120 87)"})
	@property
	def text_emerald_800(self):
		return StyleBuilder({"color": " rgb(6 95 70)"})
	@property
	def text_emerald_900(self):
		return StyleBuilder({"color": " rgb(6 78 59)"})
	@property
	def text_teal_50(self):
		return StyleBuilder({"color": " rgb(240 253 250)"})
	@property
	def text_teal_100(self):
		return StyleBuilder({"color": " rgb(204 251 241)"})
	@property
	def text_teal_200(self):
		return StyleBuilder({"color": " rgb(153 246 228)"})
	@property
	def text_teal_300(self):
		return StyleBuilder({"color": " rgb(94 234 212)"})
	@property
	def text_teal_400(self):
		return StyleBuilder({"color": " rgb(45 212 191)"})
	@property
	def text_teal_500(self):
		return StyleBuilder({"color": " rgb(20 184 166)"})
	@property
	def text_teal_600(self):
		return StyleBuilder({"color": " rgb(13 148 136)"})
	@property
	def text_teal_700(self):
		return StyleBuilder({"color": " rgb(15 118 110)"})
	@property
	def text_teal_800(self):
		return StyleBuilder({"color": " rgb(17 94 89)"})
	@property
	def text_teal_900(self):
		return StyleBuilder({"color": " rgb(19 78 74)"})
	@property
	def text_cyan_50(self):
		return StyleBuilder({"color": " rgb(236 254 255)"})
	@property
	def text_cyan_100(self):
		return StyleBuilder({"color": " rgb(207 250 254)"})
	@property
	def text_cyan_200(self):
		return StyleBuilder({"color": " rgb(165 243 252)"})
	@property
	def text_cyan_300(self):
		return StyleBuilder({"color": " rgb(103 232 249)"})
	@property
	def text_cyan_400(self):
		return StyleBuilder({"color": " rgb(34 211 238)"})
	@property
	def text_cyan_500(self):
		return StyleBuilder({"color": " rgb(6 182 212)"})
	@property
	def text_cyan_600(self):
		return StyleBuilder({"color": " rgb(8 145 178)"})
	@property
	def text_cyan_700(self):
		return StyleBuilder({"color": " rgb(14 116 144)"})
	@property
	def text_cyan_800(self):
		return StyleBuilder({"color": " rgb(21 94 117)"})
	@property
	def text_cyan_900(self):
		return StyleBuilder({"color": " rgb(22 78 99)"})
	@property
	def text_sky_50(self):
		return StyleBuilder({"color": " rgb(240 249 255)"})
	@property
	def text_sky_100(self):
		return StyleBuilder({"color": " rgb(224 242 254)"})
	@property
	def text_sky_200(self):
		return StyleBuilder({"color": " rgb(186 230 253)"})
	@property
	def text_sky_300(self):
		return StyleBuilder({"color": " rgb(125 211 252)"})
	@property
	def text_sky_400(self):
		return StyleBuilder({"color": " rgb(56 189 248)"})
	@property
	def text_sky_500(self):
		return StyleBuilder({"color": " rgb(14 165 233)"})
	@property
	def text_sky_600(self):
		return StyleBuilder({"color": " rgb(2 132 199)"})
	@property
	def text_sky_700(self):
		return StyleBuilder({"color": " rgb(3 105 161)"})
	@property
	def text_sky_800(self):
		return StyleBuilder({"color": " rgb(7 89 133)"})
	@property
	def text_sky_900(self):
		return StyleBuilder({"color": " rgb(12 74 110)"})
	@property
	def text_blue_50(self):
		return StyleBuilder({"color": " rgb(239 246 255)"})
	@property
	def text_blue_100(self):
		return StyleBuilder({"color": " rgb(219 234 254)"})
	@property
	def text_blue_200(self):
		return StyleBuilder({"color": " rgb(191 219 254)"})
	@property
	def text_blue_300(self):
		return StyleBuilder({"color": " rgb(147 197 253)"})
	@property
	def text_blue_400(self):
		return StyleBuilder({"color": " rgb(96 165 250)"})
	@property
	def text_blue_500(self):
		return StyleBuilder({"color": " rgb(59 130 246)"})
	@property
	def text_blue_600(self):
		return StyleBuilder({"color": " rgb(37 99 235)"})
	@property
	def text_blue_700(self):
		return StyleBuilder({"color": " rgb(29 78 216)"})
	@property
	def text_blue_800(self):
		return StyleBuilder({"color": " rgb(30 64 175)"})
	@property
	def text_blue_900(self):
		return StyleBuilder({"color": " rgb(30 58 138)"})
	@property
	def text_indigo_50(self):
		return StyleBuilder({"color": " rgb(238 242 255)"})
	@property
	def text_indigo_100(self):
		return StyleBuilder({"color": " rgb(224 231 255)"})
	@property
	def text_indigo_200(self):
		return StyleBuilder({"color": " rgb(199 210 254)"})
	@property
	def text_indigo_300(self):
		return StyleBuilder({"color": " rgb(165 180 252)"})
	@property
	def text_indigo_400(self):
		return StyleBuilder({"color": " rgb(129 140 248)"})
	@property
	def text_indigo_500(self):
		return StyleBuilder({"color": " rgb(99 102 241)"})
	@property
	def text_indigo_600(self):
		return StyleBuilder({"color": " rgb(79 70 229)"})
	@property
	def text_indigo_700(self):
		return StyleBuilder({"color": " rgb(67 56 202)"})
	@property
	def text_indigo_800(self):
		return StyleBuilder({"color": " rgb(55 48 163)"})
	@property
	def text_indigo_900(self):
		return StyleBuilder({"color": " rgb(49 46 129)"})
	@property
	def text_violet_50(self):
		return StyleBuilder({"color": " rgb(245 243 255)"})
	@property
	def text_violet_100(self):
		return StyleBuilder({"color": " rgb(237 233 254)"})
	@property
	def text_violet_200(self):
		return StyleBuilder({"color": " rgb(221 214 254)"})
	@property
	def text_violet_300(self):
		return StyleBuilder({"color": " rgb(196 181 253)"})
	@property
	def text_violet_400(self):
		return StyleBuilder({"color": " rgb(167 139 250)"})
	@property
	def text_violet_500(self):
		return StyleBuilder({"color": " rgb(139 92 246)"})
	@property
	def text_violet_600(self):
		return StyleBuilder({"color": " rgb(124 58 237)"})
	@property
	def text_violet_700(self):
		return StyleBuilder({"color": " rgb(109 40 217)"})
	@property
	def text_violet_800(self):
		return StyleBuilder({"color": " rgb(91 33 182)"})
	@property
	def text_violet_900(self):
		return StyleBuilder({"color": " rgb(76 29 149)"})
	@property
	def text_purple_50(self):
		return StyleBuilder({"color": " rgb(250 245 255)"})
	@property
	def text_purple_100(self):
		return StyleBuilder({"color": " rgb(243 232 255)"})
	@property
	def text_purple_200(self):
		return StyleBuilder({"color": " rgb(233 213 255)"})
	@property
	def text_purple_300(self):
		return StyleBuilder({"color": " rgb(216 180 254)"})
	@property
	def text_purple_400(self):
		return StyleBuilder({"color": " rgb(192 132 252)"})
	@property
	def text_purple_500(self):
		return StyleBuilder({"color": " rgb(168 85 247)"})
	@property
	def text_purple_600(self):
		return StyleBuilder({"color": " rgb(147 51 234)"})
	@property
	def text_purple_700(self):
		return StyleBuilder({"color": " rgb(126 34 206)"})
	@property
	def text_purple_800(self):
		return StyleBuilder({"color": " rgb(107 33 168)"})
	@property
	def text_purple_900(self):
		return StyleBuilder({"color": " rgb(88 28 135)"})
	@property
	def text_fuchsia_50(self):
		return StyleBuilder({"color": " rgb(253 244 255)"})
	@property
	def text_fuchsia_100(self):
		return StyleBuilder({"color": " rgb(250 232 255)"})
	@property
	def text_fuchsia_200(self):
		return StyleBuilder({"color": " rgb(245 208 254)"})
	@property
	def text_fuchsia_300(self):
		return StyleBuilder({"color": " rgb(240 171 252)"})
	@property
	def text_fuchsia_400(self):
		return StyleBuilder({"color": " rgb(232 121 249)"})
	@property
	def text_fuchsia_500(self):
		return StyleBuilder({"color": " rgb(217 70 239)"})
	@property
	def text_fuchsia_600(self):
		return StyleBuilder({"color": " rgb(192 38 211)"})
	@property
	def text_fuchsia_700(self):
		return StyleBuilder({"color": " rgb(162 28 175)"})
	@property
	def text_fuchsia_800(self):
		return StyleBuilder({"color": " rgb(134 25 143)"})
	@property
	def text_fuchsia_900(self):
		return StyleBuilder({"color": " rgb(112 26 117)"})
	@property
	def text_pink_50(self):
		return StyleBuilder({"color": " rgb(253 242 248)"})
	@property
	def text_pink_100(self):
		return StyleBuilder({"color": " rgb(252 231 243)"})
	@property
	def text_pink_200(self):
		return StyleBuilder({"color": " rgb(251 207 232)"})
	@property
	def text_pink_300(self):
		return StyleBuilder({"color": " rgb(249 168 212)"})
	@property
	def text_pink_400(self):
		return StyleBuilder({"color": " rgb(244 114 182)"})
	@property
	def text_pink_500(self):
		return StyleBuilder({"color": " rgb(236 72 153)"})
	@property
	def text_pink_600(self):
		return StyleBuilder({"color": " rgb(219 39 119)"})
	@property
	def text_pink_700(self):
		return StyleBuilder({"color": " rgb(190 24 93)"})
	@property
	def text_pink_800(self):
		return StyleBuilder({"color": " rgb(157 23 77)"})
	@property
	def text_pink_900(self):
		return StyleBuilder({"color": " rgb(131 24 67)"})
	@property
	def text_rose_50(self):
		return StyleBuilder({"color": " rgb(255 241 242)"})
	@property
	def text_rose_100(self):
		return StyleBuilder({"color": " rgb(255 228 230)"})
	@property
	def text_rose_200(self):
		return StyleBuilder({"color": " rgb(254 205 211)"})
	@property
	def text_rose_300(self):
		return StyleBuilder({"color": " rgb(253 164 175)"})
	@property
	def text_rose_400(self):
		return StyleBuilder({"color": " rgb(251 113 133)"})
	@property
	def text_rose_500(self):
		return StyleBuilder({"color": " rgb(244 63 94)"})
	@property
	def text_rose_600(self):
		return StyleBuilder({"color": " rgb(225 29 72)"})
	@property
	def text_rose_700(self):
		return StyleBuilder({"color": " rgb(190 18 60)"})
	@property
	def text_rose_800(self):
		return StyleBuilder({"color": " rgb(159 18 57)"})
	@property
	def text_rose_900(self):
		return StyleBuilder({"color": " rgb(136 19 55)"})



from __future__ import annotations
from .base import BaseChart
import pybi as pbi
import pybi.utils.sql as sqlUtils

from typing import TYPE_CHECKING, Optional
from pybi.core.components.reactiveComponent import EChartDatasetInfo
from .utils import merge_user_options

if TYPE_CHECKING:
    from pybi.core.dataSource import DataSourceTable


class MapChart(BaseChart):
    def __init__(self, map_name: str):
        super().__init__()
        self.map_name = map_name

    def _create_default_click_filter(self):
        pass

    def _create_options_ex(self):
        opts = super().get_options()

        geo = {
            "map": self.map_name,
            "roam": True,
            "label": {"show": True, "position": "top"},
            "itemStyle": {
                "normal": {
                    "areaColor": "#c6e2ff",
                    "borderColor": "#389dff",
                    "borderWidth": 1,
                },
                "emphasis": {
                    "areaColor": "#337ecc",
                    "shadowOffsetX": 0,
                    "shadowOffsetY": 0,
                    "shadowBlur": 5,
                    "borderWidth": 0,
                    "shadowColor": "rgba(0, 0, 0, 0.5)",
                },
            },
        }

        base_opt = {"xAxis": None, "yAxis": None, "series": [], "geo": geo}
        base_opt.update(opts)

        series_config = {
            "type": "map",
            "geoIndex": 0,
        }

        base_opt["series"].append(series_config)

        # opts["tooltip"] = {"trigger": "item"}

        return base_opt, self._updateInfos, [self.map_name]



from __future__ import annotations
from .base import BaseChart
import pybi as pbi

from typing import TYPE_CHECKING, Optional


if TYPE_CHECKING:
    from pybi.core.dataSource import DataSourceTable


class ScatterChart(BaseChart):
    def __init__(
        self,
        data: DataSourceTable,
        x: str,
        y: str,
        color: Optional[str] = None,
        size: Optional[str] = None,
        agg="round(avg(${}),2)",
    ):
        super().__init__()
        self.data = data
        self.x = x
        self.y = y
        self.color = color
        self.size = size
        self.agg = agg
        self._series_configs = {}

    def _create_default_click_filter(self):
        # TODO:
        self.click_filter("x", self.data, self.x)

    def _create_options_ex(self):
        opts = super().get_options()

        base_opt = {
            "xAxis": {},
            "yAxis": {},
            "series": [],
        }
        base_opt.update(opts)

        series_config = {"type": "scatter", "symbolSize": 15}
        series_config.update(self._series_configs)

        sql = None
        if self.color:
            sql = pbi.sql(
                f"select {self.x},{self.y},{self.color} from {self.data}"
            ).split_group(
                self.color,
                """
                const need_fields = fields.slice(0,2)
                const data = rows.map(r=> need_fields.map(f=> r[f]))
                return {data,type:'scatter',name:key,symbolSize:15}""",
            )

            base_opt["series"] = sql
        else:
            sql = pbi.sql(f"select {self.x},{self.y} from {self.data}").toflatlist()
            series = {"type": "scatter", "symbolSize": 15, "data": sql}
            base_opt["series"].append(series)

        opts["tooltip"] = {"trigger": "item"}

        return base_opt, self._updateInfos, []



from __future__ import annotations
from dataclasses import field
from multiprocessing.sharedctypes import Value
from typing import Dict, List, Union
from copy import deepcopy
import re


def user_opts_handler(obj: dict):
    stack = []
    for key, value in obj.items():
        stack.append((key, value))

    def getNext():
        path, data = stack.pop()
        return path, data

    def pushData(start_path: str, newData: dict):
        for key, value in newData.items():
            stack.append((f"{start_path}.{key}", value))

    def shouldStop():
        return len(stack) <= 0

    return getNext, pushData, shouldStop

def base_opt_handler(opts: dict):
    base_opts = deepcopy(opts)
    
    def pathGet(dictionary: dict, path: str):
        #按照[ . ]切割字符串，因为会有空值和数字字符串，要进行一下判断
        if ']' not in path:
            path = re.sub(r'([^.]+)', r'\1'+'[0]', path, count=1)
        path_list = re.split('[\[,\],.]',path)

        try:
            path_list.remove('')
        except ValueError:
            pass

        len_path_list = len(path_list)

        for num, item in enumerate(path_list):
            if item.isdigit():
                item = int(item)
            if num == len_path_list - 1:
                dictionary_last = dictionary
            try:
                dictionary = dictionary[item]
            except KeyError:
                return False, dictionary, item
                
        return True, dictionary_last, item

    def try_update(path: str, data):
        dictionary = base_opts
        isdict, dictionary, item = pathGet(dictionary, path)
        #如果路径不存在 或 存在，但是data不是一个字典，直接新增，返回True
        
        if not isinstance(data, dict):
            dictionary[item] = data
            return True
        #如果路径存在，且data是字典，则不更新，返回False
        else:
            if isdict:
                return False
            else:
                dictionary[item] = data
                return True
    def get_opts():
        return base_opts
    
    return try_update, get_opts

def merge_user_options(base_opts: Dict, user_opts: Dict):
    
    getNext, pushData, shouldStop = user_opts_handler(user_opts)
    try_update, get_opts = base_opt_handler(base_opts)

    while not shouldStop():
        path, data = getNext()

        if not try_update(path, data):
            pushData(path, data)
        
    return get_opts()


from __future__ import annotations
from typing import TYPE_CHECKING, List, Optional, Union, Dict


from pybi.utils.data_gen import Jsonable, get_global_id
from pybi.utils.helper import style_text2dict
from .componentTag import ComponentTag
from pybi.core.styles.styles import StyleBuilder

if TYPE_CHECKING:
    from pybi.core.styles.styles import Style
    from pybi.core.sql import SqlWrapper


class Component(Jsonable):
    def __init__(self, tag: ComponentTag) -> None:
        self.id = get_global_id()
        self.tag = tag
        self._debugTag: Optional[str] = None
        self._styles: List[Style] = []
        self.visible: Union[bool, SqlWrapper] = True
        self.__gridArea: str = ""
        self._props = {}
        self.classes = []

    def set_props(self, props: Dict):
        self._props.update(props)
        return self

    def set_debugTag(self, tag: str):
        self._debugTag = tag
        return self

    def set_classes(self, value: str):
        """
        set_classes('text-primary bg-positive')
        """
        values = (v for v in value.split(" ") if v)
        self.classes.extend(values)
        return self

    def set_gridArea(self, area_name: str):
        self.__gridArea = area_name
        return self

    def set_style(self, content: str):
        """添加样式

        Args:
            content (str): 样式字符串,多个样式用分号相隔

        ### 示例
        >>> .set_style('color: #6E93D6; font-size: 200%; font-weight: 300')
        """
        styles = style_text2dict(content)

        self._styles.append(StyleBuilder(styles))
        return self

    def _to_json_dict(self):
        data = super()._to_json_dict()
        styles_dict = {}
        for style in self._styles:
            styles_dict.update(style._get_styles_dict())
        data["styles"] = styles_dict

        if isinstance(self.visible, bool) and self.visible == True:
            del data["visible"]

        if self.__gridArea:
            data["gridArea"] = self.__gridArea

        data["debugTag"] = self._debugTag
        data["props"] = self._props
        data["classes"] = list(dict.fromkeys(data["classes"]).keys())

        return data

    def __add__(self, other: Style):
        self._styles.append(other)
        return self

    def __hash__(self) -> int:
        return hash(self.id)

    def __eq__(self, __o: object) -> bool:
        if isinstance(__o, self.__class__):
            return hash(__o.id) == hash(self.id)

        return False



from __future__ import annotations
from typing import TYPE_CHECKING, List, Dict


from pybi.core.components import ComponentTag
from .base import ReactiveComponent


class NumberSlider(ReactiveComponent):
    def __init__(self, where_expr: str) -> None:
        super().__init__(ComponentTag.NumberSlider)
        self.whereExpr = where_expr

    def set_props(self, props: Dict):
        """
        [slider props](https://element-plus.gitee.io/zh-CN/component/slider.html#%E5%B1%9E%E6%80%A7)
        e.g
        >>> .add_numberSlider(...).set_props({'show-stops':True})
        """
        return super().set_props(props)



from __future__ import annotations
from typing import TYPE_CHECKING, List, Dict


from pybi.core.components import ComponentTag
from .base import ReactiveComponent


if TYPE_CHECKING:
    from pybi.core.sql import SqlInfo


class Input(ReactiveComponent):
    def __init__(self, where_expr: str) -> None:
        super().__init__(ComponentTag.Input)
        self.whereExpr = where_expr

    def set_props(self, props: Dict):
        """
        [input props](https://element-plus.gitee.io/zh-CN/component/input.html#attributes)
        e.g
        >>> .add_input(...).set_props({'placeholder':'my define placeholder'})
        """
        return super().set_props(props)



from __future__ import annotations
from .base import BaseChart
import pybi as pbi
import pybi.utils.sql as sqlUtils

from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from pybi.core.dataSource import DataSourceTable


class LineChart(BaseChart):
    def __init__(
        self,
        data: DataSourceTable,
        x: str,
        y: str,
        order: str,
        color: Optional[str] = None,
        agg="round(avg(${}),2)",
    ):
        super().__init__()
        self.data = data
        self.x = x
        self.y = y
        self.order = order or x
        self.color = color
        self.agg = agg
        self._series_configs = {}
        self._xAxis = {}

    def stack(self):
        self._series_configs["stack"] = "Total"
        return self

    def area(self):
        """
        线以下区域填充颜色,使其成为区域图
        """
        self._series_configs["areaStyle"] = {}
        return self

    def _create_default_click_filter(self):
        self.click_filter("x", self.data, self.x)

    def set_xAxis(self, value):
        self._xAxis = value
        return self

    def _create_options_ex(self):
        opts = super().get_options()

        base_opt = {
            "xAxis": [
                {
                    "type": "",
                    "data": [],
                }
            ],
            "yAxis": [{"type": ""}],
            "series": [],
        }
        base_opt.update(opts)

        sql = ""
        agg_field = f"{sqlUtils.apply_agg(self.agg, self.y)} as {self.y}"
        if self.color:
            sql = f"select {self.x},{agg_field},{self.color},{self.order} from {self.data} group by {self.color},{self.x}"
        else:
            sql = f"select {self.x},{agg_field},{self.order} from {self.data} group by {self.x}"

        dv = pbi.set_dataView(sql)

        if self.color:
            base_opt["series"] = pbi.sql(
                f"select {self.x},{self.y},{self.color} from {dv} order by {self.order}"
            ).split_group(
                self.color,
                """
                const data = rows.map(r=> [r[fields[0]],r[fields[1]]])
                return {id:key,data,type:'line',name:key,universalTransition: {enabled: true}}""",
            )
        else:
            base_opt["series"].append(
                {
                    "name": self.x,
                    "id": self.x,
                    "universalTransition": {"enabled": True, "divideShape": "clone"},
                    "type": "line",
                    "data": pbi.sql(
                        f"select {self.y} as value,{self.x} as groupId from {dv} order by {self.order}"
                    ),
                }
            )

        catAxis = base_opt["xAxis"][0]
        valueAxis = base_opt["yAxis"][0]

        catAxis["data"] = pbi.sql(
            f"select distinct {self.x} from {dv} order by {self.order}"
        ).toflatlist()
        catAxis.update(self._xAxis)
        # catAxis["axisLabel"] = {"formatter": "{value}月"}
        catAxis["boundaryGap"] = False
        valueAxis["type"] = "value"

        opts["tooltip"] = {
            "trigger": "axis",
            "axisPointer": {"type": "cross", "label": {"backgroundColor": "#6a7985"}},
        }

        return base_opt, self._updateInfos, []



from __future__ import annotations
from typing import TYPE_CHECKING, List, Optional, Union, Dict
from pybi.core.components.component import Component


from pybi.utils.data_gen import Jsonable, get_global_id
from .componentTag import ComponentTag


class Mermaid(Component):
    def __init__(self, graph: str, name) -> None:
        super().__init__(ComponentTag.Mermaid)
        # self.graph = graph
        self.__graph_infos = [{"graph": graph, "name": name}]
        self.__relationships = []

    def add_graph(self, graph: str, name: str):
        self.__graph_infos.append({"graph": graph, "name": name})
        return self

    def set_relationship(self, rel_str: str):
        '''
        例子：
        ```python
        mm = pbi.add_mermaid(
            """
        graph TB
        FullFirstSquad-->StripedFirstSquad
        """,'main')

        mm.add_graph(
            """
            flowchart LR
        A[Hard] -->|Text| B(Round)
        B --> C{Decision}
        C -->|One| D[Result 1]
        C -->|Two| E[Result 2]
        """
            "g2",
        )

        # main 流程图的节点 FullFirstSquad，点击后跳转到 g2 流程图
        mm.set_relationship("main.FullFirstSquad>g2")
        ```


        '''
        # parent = 'g1'
        # other = 'g>g2'
        parent, other = rel_str.split(".")
        node, child = other.split(">")

        return self.__set_relationship(parent, node, child)

    def __set_relationship(
        self, parent_graph_name: str, node_name: str, child_graph_name: str
    ):
        self.__relationships.append(
            {"parent": parent_graph_name, "node": node_name, "child": child_graph_name}
        )
        return self

    def _to_json_dict(self):
        data = super()._to_json_dict()
        data["graphInfos"] = self.__graph_infos

        data["relationships"] = self.__relationships

        return data



from __future__ import annotations
from typing import TYPE_CHECKING, List, Dict


from pybi.core.components import ComponentTag
from .base import SingleReactiveComponent


if TYPE_CHECKING:
    from pybi.core.sql import SqlInfo


class Slicer(SingleReactiveComponent):
    def __init__(self, sql: SqlInfo) -> None:
        super().__init__(ComponentTag.Slicer, sql)
        self.title = ""
        self.multiple = True
        self.__hidden_title = None
        self.__default_selected = None

    def set_title(self, title: str):
        self.title = title
        return self

    def _set_default_selected(self, value=True):
        """
        首次显示时，默认选中第一项
        """
        self.__default_selected = value
        return self

    def set_hiddenTitle(self, value=True):
        """
        不显示标题
        """
        self.__hidden_title = value
        return self

    def set_multiple(self, multiple: bool):
        self.multiple = multiple
        return self

    def set_props(self, props: Dict):
        """
        [slicer props](https://element-plus.gitee.io/zh-CN/component/select.html#select-attributes)
        e.g
        >>> .add_slicer(...).set_props({'placeholder':'my define placeholder'})
        """
        return super().set_props(props)

    def _to_json_dict(self):
        data = super()._to_json_dict()

        if self.__hidden_title:
            data["hiddenTitle"] = self.__hidden_title

        if self.__default_selected:
            data["defaultSelected"] = self.__default_selected
        return data



import pandas as pd
from typing import Any, Callable, TypeVar, List, Iterable, Generator, Union

T = TypeVar("T")


def flatten(
    obj: T, children_fn: Callable[[T], Iterable[T]]
) -> Generator[T, None, None]:
    stack = [obj]

    while 1:
        if len(stack) <= 0:
            break
        target = stack.pop()
        yield target
        children = children_fn(target)
        for child in children:
            stack.append(child)


def df_na2none(df: pd.DataFrame):
    return df.where(pd.notnull(df), None)


def df2object_dict(df: pd.DataFrame):
    return df_na2none(df).to_dict("records")


def df2array_dict(df: pd.DataFrame):
    return df_na2none(df).values.tolist()


def value2code(v: Any):
    if v is None:
        return "null"
    return str(v) if not isinstance(v, str) else f"'{v}'"


def style_text2dict(style: str):
    pairs = style.split(";")
    items = (s.split(":") for s in pairs if s)
    items = (item for item in items if len(item) == 2)
    items = {name.strip(): value.strip() for name, value in items}
    return items
